SystemOrganization addCategory: #Prolog!
SystemOrganization addCategory: #'Prolog-Predicate'!
SystemOrganization addCategory: #'Prolog-Test'!

TestCase subclass: #AncestorTest
	instanceVariableNames: 'k'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prolog-Test'!

!AncestorTest methodsFor: 'running' stamp: 'HwaJong 10/23/2012 10:34'!
setUp
super setUp.
k := PrologKnowledgeBase new.

k fact: #father data: {#zeb. #johnBoySr}.
k fact: #father data: {#johnBoySr. #johnBoyJr}.

k rule: #ancestor definition: [:x :y| k query: #father data: {x.y}].
k rule: #ancestor dataSize: 2 definition: [:x :y :m| (k query: #ancestor data: {x.m}) and: [k query: #ancestor data: {m.y}]].! !

!AncestorTest methodsFor: 'tests' stamp: 'HwaJong 10/22/2012 23:46'!
testFather
self assert: (k query: #father data: {#zeb. #johnBoySr}).
self assert: (k query: #father data: {#johnBoySr. #johnBoyJr}).

self deny: (k query: #father data: {#zeb. #johnBoyJr}).
self deny: (k query: #father data: {#johnBoyJr. #johnBoySr}).! !

!AncestorTest methodsFor: 'tests' stamp: 'HwaJong 10/23/2012 02:13'!
testFatherIsAncestor
self assert: (k query: #ancestor data: {#zeb. #johnBoySr}).
self assert: (k query: #ancestor data: {#johnBoySr. #johnBoyJr}).

self assert: (k query: #ancestor data: {#zeb. #johnBoyJr}).
self deny: (k query: #ancestor data: {#johnBoyJr. #johnBoySr}).! !

!AncestorTest methodsFor: 'tests' stamp: 'HwaJong 10/22/2012 23:48'!
testGrandFatherIsAncestor
self assert: (k query: #ancestor data: {#zeb. #johnBoyJr}).! !

TestCase subclass: #CheeseAndSheepTest
	instanceVariableNames: 'k'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prolog-Test'!

!CheeseAndSheepTest methodsFor: 'running' stamp: 'HwaJong 10/23/2012 02:53'!
setUp
super setUp.
k := PrologKnowledgeBase new.

k fact: #likes data: {#wallace. #cheese}.
k fact: #likes data: {#grommit. #cheese}.
k fact: #likes data: {#wendolene. #sheep}.
! !

!CheeseAndSheepTest methodsFor: 'tests' stamp: 'HwaJong 10/23/2012 02:53'!
testAllLike
| searchResult |
searchResult := k queryWhat: #likes data: {#P. #O}.
self assert: searchResult size equals: 3.
self assert: (searchResult at: 1) equals: {#wallace. #cheese}.
self assert: (searchResult at: 2) equals: {#grommit. #cheese}.
self assert: (searchResult at: 3) equals: {#wendolene. #sheep}.! !

!CheeseAndSheepTest methodsFor: 'tests' stamp: 'HwaJong 10/23/2012 02:53'!
testEmptyResultIfNoMatch
| searchResult |
searchResult := k queryWhat: #likes data: {1. #W}.
self assert: searchResult size equals: 0.! !

!CheeseAndSheepTest methodsFor: 'tests' stamp: 'HwaJong 10/23/2012 02:53'!
testFactsAreTrue
self assert: (k query: #likes data: {#wallace. #cheese}).! !

!CheeseAndSheepTest methodsFor: 'tests' stamp: 'HwaJong 10/23/2012 02:53'!
testFalseIfNotAFact
self deny: (k query: #likes data: {#wallace. #sheep}).! !

!CheeseAndSheepTest methodsFor: 'tests' stamp: 'HwaJong 10/23/2012 02:53'!
testWhoLikesCheese
| searchResult |
searchResult := k queryWhat: #likes data: {#X. #cheese}.
self assert: searchResult size equals: 2.
self assert: (searchResult at: 1) equals: {#wallace}.
self assert: (searchResult at: 2) equals: {#grommit}.! !

TestCase subclass: #FriendTest
	instanceVariableNames: 'k'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prolog-Test'!

!FriendTest methodsFor: 'running' stamp: 'HwaJong 10/23/2012 10:32'!
setUp
super setUp.
k := PrologKnowledgeBase new.

k fact: #likes data: {#wallace. #cheese}.
k fact: #likes data: {#grommit. #cheese}.
k fact: #likes data: {#wendolene. #sheep}.
	
k rule: #friend
	dataSize: 2
	definition: [:p1 :p2 :thing| p1 ~= p2 and: [(k query: #likes data: {p1. thing}) and: [k query: #likes data: {p2. thing}]]].! !

!FriendTest methodsFor: 'tests' stamp: 'HwaJong 10/23/2012 02:52'!
testEnlist
|list|
list := k queryWhat: #friend data: {#A. #B}. 
self assert: list size equals: 2.
self assert: (list at: 1) equals: {#wallace. #grommit}.
self assert: (list at: 2) equals: {#grommit. #wallace}.! !

!FriendTest methodsFor: 'tests' stamp: 'HwaJong 10/23/2012 02:52'!
testFriends
self assert: (k query: #friend data: {#wallace. #grommit}).! !

!FriendTest methodsFor: 'tests' stamp: 'HwaJong 10/23/2012 02:52'!
testNotFriends
self deny: (k query: #friend data: {#wallace. #wendolene}).! !

TestCase subclass: #MapColoringTest
	instanceVariableNames: 'k'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prolog-Test'!

!MapColoringTest methodsFor: 'running' stamp: 'HwaJong 10/23/2012 10:34'!
setUp
super setUp.
k := PrologKnowledgeBase new.
k fact: #different data: {#red. #green}.
k fact: #different data: {#red. #blue}.
k fact: #different data: {#green. #red}.
k fact: #different data: {#green. #blue}.
k fact: #different data: {#blue. #red}.
k fact: #different data: {#blue. #green}.

k rule: #coloring
	definition: [:alabama :mississippi :georgia : tennesee :florida|
	(k query: #different data: {mississippi. tennesee}) and: [
	(k query: #different data: {mississippi. alabama}) and: [
	(k query: #different data: {alabama. tennesee}) and: [
	(k query: #different data: {alabama. mississippi}) and: [
	(k query: #different data: {alabama. georgia}) and: [
	(k query: #different data: {alabama. florida}) and: [
	(k query: #different data: {georgia. florida}) and: [
	(k query: #different data: {georgia. tennesee})]]]]]]] ].
! !

!MapColoringTest methodsFor: 'tests' stamp: 'HwaJong 10/22/2012 19:19'!
testResult
|result|
result := k queryWhat: #coloring data: {#A. #B. #C. #D. #E}.
self assert: result size equals: 6.
self assert: (result includes: {#blue. #red. #red. #green. #green})! !

TestCase subclass: #SimpleRuleTest
	instanceVariableNames: 'k'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prolog-Test'!

!SimpleRuleTest methodsFor: 'running' stamp: 'HwaJong 10/23/2012 10:33'!
setUp
super setUp.
k := PrologKnowledgeBase new.

k fact: #likes data: {#wallace. #cheese}.
k fact: #likes data: {#grommit. #cheese}.
k fact: #likes data: {#wendolene. #sheep}.

k rule: #cheeseLover dataSize: 1 definition: [:x| k query: #likes data: {x. #cheese}].
! !

!SimpleRuleTest methodsFor: 'tests' stamp: 'HwaJong 10/22/2012 17:43'!
testAllCheeseLover
|searchResult|
searchResult := k queryWhat: #cheeseLover data: {#Person}.
self assert: searchResult size equals: 2.
self assert: (searchResult at: 1) equals: {#wallace}.
self assert: (searchResult at: 2) equals: {#grommit}.! !

!SimpleRuleTest methodsFor: 'tests' stamp: 'HwaJong 10/22/2012 17:42'!
testFalse
self deny: (k query: #cheeseLover data: {#wendolene}).
! !

!SimpleRuleTest methodsFor: 'tests' stamp: 'HwaJong 10/22/2012 17:41'!
testTrue
self assert: (k query: #cheeseLover data: {#wallace}).
! !

!ByteSymbol methodsFor: '*Prolog' stamp: 'HwaJong 10/22/2012 17:12'!
representsPrologVariable
^self first isUppercase! !

!Object methodsFor: '*Prolog' stamp: 'HwaJong 10/22/2012 17:01'!
isPrologVariable
^false! !

!Object methodsFor: '*Prolog' stamp: 'HwaJong 10/22/2012 17:13'!
representsPrologVariable
^false! !

Object subclass: #PrologKnowledgeBase
	instanceVariableNames: 'entries flatGrandDomain'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prolog'!

!PrologKnowledgeBase methodsFor: 'accessing' stamp: 'HwaJong 10/22/2012 15:27'!
entryAtPredicateName: aSymbol 
	^entries at: aSymbol ifAbsentPut: [OrderedCollection new] ! !

!PrologKnowledgeBase methodsFor: 'predicating' stamp: 'HwaJong 10/23/2012 02:44'!
fact: aSymbol data: anArray 
	|f|
	f := Fact factName: aSymbol data: anArray.
	(self entryAtPredicateName: aSymbol) addLast: f.
	flatGrandDomain addAll: f data.! !

!PrologKnowledgeBase methodsFor: 'accessing' stamp: 'HwaJong 10/22/2012 14:08'!
flatGrandDomain
^flatGrandDomain! !

!PrologKnowledgeBase methodsFor: 'initialize-release' stamp: 'HwaJong 10/22/2012 14:06'!
initialize
super initialize.
entries := Dictionary new.
flatGrandDomain := Set new.! !

!PrologKnowledgeBase methodsFor: 'accessing' stamp: 'HwaJong 10/22/2012 13:37'!
predicateNames
^entries keys! !

!PrologKnowledgeBase methodsFor: 'query' stamp: 'HwaJong 10/22/2012 15:28'!
query: aSymbol data: anArray 
	(entries includesKey: aSymbol) ifFalse: [^false].
	(entries at: aSymbol) detect: [:each| each isTrueWithName: aSymbol data: anArray ] ifNone: [^false].
	^true! !

!PrologKnowledgeBase methodsFor: 'query' stamp: 'HwaJong 10/22/2012 18:50'!
queryWhat: aSymbol data: anArray 
^(QueryWhat knowledgeBase: self predicateName: aSymbol data: anArray) value.! !

!PrologKnowledgeBase methodsFor: 'predicating' stamp: 'HwaJong 10/23/2012 10:37'!
rule: aSymbol dataSize: anInteger definition: definitionClosure
	TTTMultiDimensionalDo new
		domains: (Array new: (definitionClosure numArgs) withAll: (self flatGrandDomain asArray));
		block: [:params| (definitionClosure valueWithArguments: params) ifTrue: [self fact: aSymbol data: (params first: anInteger) copy] ];
		run.! !

!PrologKnowledgeBase methodsFor: 'predicating' stamp: 'HwaJong 10/23/2012 10:36'!
rule: aSymbol definition: definitionClosure 
self rule: aSymbol dataSize: definitionClosure numArgs definition: definitionClosure! !

Object subclass: #PrologPredicate
	instanceVariableNames: 'predicateName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prolog-Predicate'!

PrologPredicate subclass: #Fact
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prolog-Predicate'!

!Fact classSide methodsFor: 'as yet unclassified' stamp: 'HwaJong 10/22/2012 15:30'!
factName: aSymbol data: anArray 
^self new
	factName: aSymbol;
	data: anArray! !

!Fact methodsFor: 'accessing' stamp: 'HwaJong 10/21/2012 22:11'!
data

	^ data! !

!Fact methodsFor: 'accessing' stamp: 'HwaJong 10/21/2012 21:46'!
data: anArray 
data := anArray! !

!Fact methodsFor: 'accessing' stamp: 'HwaJong 10/22/2012 15:29'!
factName: aSymbol 
predicateName := aSymbol ! !

!Fact methodsFor: 'testing' stamp: 'HwaJong 10/22/2012 15:29'!
isTrueWithName: aSymbol data: anArray 
	predicateName = aSymbol ifFalse: [^false].
	data = anArray ifFalse: [^false].
	^true   ! !

!PrologPredicate methodsFor: 'accessing' stamp: 'HwaJong 10/22/2012 13:21'!
predicateName
	^ predicateName! !

Object subclass: #PrologVariable
	instanceVariableNames: 'collectedCases'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prolog'!

!PrologVariable methodsFor: 'testing' stamp: 'HwaJong 10/22/2012 17:01'!
isPrologVariable
^true! !

Object subclass: #QueryWhat
	instanceVariableNames: 'knowledgeBase queryFact collectedCases'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prolog'!

!QueryWhat classSide methodsFor: 'as yet unclassified' stamp: 'HwaJong 10/22/2012 18:51'!
knowledgeBase: aPrologKnowledgeBase predicateName: aSymbol data: anArray 
|q3 v|
v := PrologVariable new.
q3 := self new
	knowledgeBase: aPrologKnowledgeBase;
	queryFact: (Fact factName: aSymbol data: (anArray collect: [:each| each representsPrologVariable ifTrue: [v] ifFalse: [each]])).
^q3.

! !

!QueryWhat methodsFor: 'as yet unclassified' stamp: 'HwaJong 10/22/2012 16:48'!
collectResultFromArray: anArray 
self collectedCases addLast: (self queryIndicies collect: [:index| anArray at: index])! !

!QueryWhat methodsFor: 'accessing' stamp: 'HwaJong 10/22/2012 16:45'!
collectedCases
collectedCases ifNil: [collectedCases := OrderedCollection new].
^collectedCases ! !

!QueryWhat methodsFor: 'accessing' stamp: 'HwaJong 10/22/2012 16:38'!
knowledgeBase: aPrologKnowledgeBase 
knowledgeBase := aPrologKnowledgeBase ! !

!QueryWhat methodsFor: 'accessing' stamp: 'HwaJong 10/22/2012 16:38'!
queryFact: aFact 
queryFact := aFact ! !

!QueryWhat methodsFor: 'accessing' stamp: 'HwaJong 10/22/2012 17:01'!
queryIndicies
^(1 to: queryFact data size) select: [:index| (queryFact data at: index) isPrologVariable ].! !

!QueryWhat methodsFor: 'as yet unclassified' stamp: 'HwaJong 10/22/2012 17:01'!
run

| md anArray aSymbol |
anArray := queryFact data.
aSymbol := queryFact predicateName.
md := TTTMultiDimensionalDo new
	domains: ((1 to: anArray size) collect: [:index| (anArray at: index) isPrologVariable ifTrue: [knowledgeBase flatGrandDomain asArray] ifFalse: [{anArray at: index}]] );
	block: [:array| (knowledgeBase query: aSymbol data: array)
			ifTrue: [ self collectResultFromArray: array ].
		];
	run.! !

!QueryWhat methodsFor: 'evaluating' stamp: 'HwaJong 10/22/2012 16:52'!
value
self run.
^self collectedCases! !
